/*
  Crochet Cat Lamp

  created 20 Apr 2024
  by Katie Hsieh

  https://hackaday.io/project/195704-crochet-cat-lamp
*/

#include <Arduino_BMI270_BMM150.h>
#include <Adafruit_NeoPixel.h>

const int ledPin = 14;
const int numPixels = 4;
const int buttonPinL = 12;
const int buttonPinR = 11;

Adafruit_NeoPixel pixels(numPixels, ledPin, NEO_GRB + NEO_KHZ800);

int degreesX = 0;
int degreesY = 0;
int buttonStateL = HIGH;
int buttonStateR = HIGH;
int brightness = 100;
int red = 255;
int green = 255;
int blue = 255;
int redSave = 255;
int greenSave = 255;
int blueSave = 255;
bool colorMode = true;

void setup() {
  pinMode(buttonPinL, INPUT_PULLUP);
  pinMode(buttonPinR, INPUT_PULLUP);

  Serial.begin(9600);
  Serial.println("Started");

  if (!IMU.begin()) {
    Serial.println("Failed to initialize IMU!");
    while (1);
  }

  Serial.print("Accelerometer sample rate = ");
  Serial.print(IMU.accelerationSampleRate());
  Serial.println(" Hz");

  pixels.begin(); // Initialize NeoPixel strip object
}

void loop() {
  // Accelerometer controls color
  float x, y, z;
  if (IMU.accelerationAvailable()) {
    red = 0;
    green = 0;
    blue = 0;
    IMU.readAcceleration(x, y, z);
    if (x > 0.1) {
      x = 100 * x;
      degreesX = map(x, 0, 97, 0, 90);
      // Serial.print("Tilting left ");
      // Serial.print(degreesX);
      // Serial.println("  degrees");
      blue = 255 - (degreesX * 3);
    }
    if (x < -0.1) {
      x = 100 * x;
      degreesX = map(x, 0, -100, 0, 90);
      // Serial.print("Tilting right ");
      // Serial.print(degreesX);
      // Serial.println("  degrees");
      green = 255 - (degreesX * 3);
    }
    if (y > 0.1) {
      y = 100 * y;
      degreesY = map(y, 0, 97, 0, 90);
      // Serial.print("Tilting up ");
      // Serial.print(degreesY);
      // Serial.println("  degrees");
      red = 255 - (degreesY * 3);
    }
    if (y < -0.1) {
      y = 100 * y;
      degreesY = map(y, 0, -100, 0, 90);
      // Serial.print("Tilting down ");
      // Serial.print(degreesY);
      // Serial.println("  degrees");
      green = 255 - (degreesY * 3);
      blue = 255 - (degreesY * 3);
    }
    if (x < 0.1 && x > -0.1 && y < 0.1 && y > -0.1) {
      red = 255;
      green = 255;
      blue = 255;
    }
  }

  // Left button controls brightness
  if (buttonStateL == HIGH && digitalRead(buttonPinL) == LOW) {
    buttonStateL = LOW;
    if (brightness == 100) {
      brightness = 0;
    }
    else {
      brightness += 25;
    }
    Serial.print("Brightness: ");
    Serial.println(brightness);
  }
  else if (buttonStateL == LOW && digitalRead(buttonPinL) == HIGH) {
    buttonStateL = HIGH;
  }
  
  // Right button toggles color mode
  if (buttonStateR == HIGH && digitalRead(buttonPinR) == LOW) {
    buttonStateR = LOW;
    colorMode = !colorMode;
    Serial.print("Color Mode: ");
    if (colorMode) {
      Serial.println("tilt");
    }
    else {
      Serial.println("fix");
      redSave = red;
      greenSave = green;
      blueSave = blue;
    }
  }
  else if (buttonStateR == LOW && digitalRead(buttonPinR) == HIGH) {
    buttonStateR = HIGH;
  }

  // Update pixels
  pixels.setBrightness(brightness);
  for (int i = 0; i < numPixels; i++) {
    if (colorMode) {
      pixels.setPixelColor(i, pixels.Color(red, green, blue));
    }
    else {
      pixels.setPixelColor(i, pixels.Color(redSave, greenSave, blueSave));
    }
  }
  pixels.show();
}
